% ZDO - cviceni 4 - Fourierova transformace
% Furierova transformace je dulezity nastroj pri zpracovani obrazu. Pouziva
% se k dekompozici obrazu na jeho sin a cos slozky. Vystup transformace
% reprezentuje obraz ve furierove nebo frekvencnim prostoru, zatimco
% vstupni obraz je v realnem prostoru. Ve fourierove obraze kazdy bod
% reprezentuje jednotlivou frekvenci obsazenou v realnem obraze. Fourierova
% transformace se pouziva v siroke oblasti aplikaci, jako je analyza
% obrazu, fitrace obrazu, rekonstrukce obrazu a komprese obrazu

% Prakticka ukazka
% Fourierova transformace se pouziva pokud chceme ziskat geometricke
% charakteristiky obrazu. Protoze obraz je ve fourierovi rozlozen na
% sinusovky je prohlizeni a zpracovani urcitych frekvenci jednoduche. 
% Ve vetsine realizaci je Fourieruv obraz posunut (fftshift) tak aby
% hodnoty odpovidajici nejvetsim frekvencim byly zobrazeny ve stredu 
% obrazu. Cim dale od stredu je bod umisten tim vetsi je jeho 
% korespondujici frekvence.

% ===============   Zacneme ==============
% aplikujeme fourierov transformaci a zobrazime si velikost (amplitudu)
% spoctenou z komplexniho vysledku transformace.
close all;
im=rgb2gray(imread('ctverce.jpg'));
figure;imshow(im);
F_im=fft2(im);
figure;mesh(real(F_im)); title('Realna slozka');
figure;mesh(imag(F_im)); title('Imaginarni slozka');
amp=abs(F_im); % Amplituda sqrt(Re^2 + Im^2);
figure;mesh(amp); title('Amlitudove spektrum')
figure;mesh(fftshift(amp)); title('Shifted - Amlitudove spektrum');
%Z obrazku muzeme videt ze hodnota ve stredu je nejvetsi slozka obrazku.
%Ostatni hodnoty jsou moc male aby byly zobrazeny. Proto zobrazujem pomoci
%logaritmicke transformace.
ampLog=log(amp);
figure;mesh(ampLog);title('logaritmicky zobrazene Amlitudove spektrum')
figure;mesh(fftshift(ampLog)); title('logaritmicky zobrazene shifted - Amlitudove spektrum')
figure;imshow(ampLog); title('logaritmicky zobrazene Amlitudove spektrum bez mezi');
figure;imshow(ampLog,[min(ampLog(:)) max(ampLog(:))]); title('logaritmicky zobrazene Amlitudove spektrum'); colormap(jet);
figure;imshow(fftshift(ampLog),[min(ampLog(:)) max(ampLog(:))]); title('logaritmicky zobrazene Shifted - Amlitudove spektrum'); colormap(jet);
% Vysledek ukazuje ze obraz obsahuje slozky vsech frekvenci ale jejich
% velikost je tim mensi cim vetsi je frekvence. Z toho vypliva ze nizke
% frekvence obsahuji vice informaci o obrazu nez  vetsi frekvence. 
faze=atan(imag(F_im)./real(F_im)); 
figure; imshow(faze);title('fazova charakteristika');colormap(jet);
figure; imshow(log(faze));title('logaritmicky zobrazeba fazova charakteristika');colormap(jet);
% Na obrazku vidime fourierovu fazovou charkteristiku. Hodnota kazdeho bodu
% urcuje fazi korespondujici frekvence. Fazova charakteristika neobsahuje
% mnoho novych informaci o strukture realneho obrazu. Proto se dale budem zaobirat 
% pouze amplitudovou charakteristikou. POZOR ALE pokud aplikujeme inverzni
% transformaci a ignorujeme fazy ziskame:
imshow(abs(ifft2(amp)),[]); title('Inverzni zobrazeni - zanedbana faze')
% I kdyz tento obraz obsauje stejne frekvence i stejny pocet frekvenci jako
% original je tento obraz porusen. Informace o fazi je kriticky potrebna pro 
% spravnou rekonstrukci obrazu.

% =========================== FILTRACE ==========================
% Fourierova transformace muze byt pouzita i pro filtrovani.
% Operace Konvoluce se ve furierove prostoru zmeni v operaci nasobeni (Konvolucni teorem)
% Vytvorime si kruhovy filtr a pronasobenim s fourierovym obrazem.
%==== vytvor filtr dolni propust
D=zeros(size(im));
r=20; % polomer
mask=fspecial('disk',r);
[m,n]=size(im);
x=round(m/2)-r;
y=round(n/2)-r;

D(x:x+size(mask,1)-1, y:y+size(mask,1)-1) = mask>0;
imFilt=F_im.*ifftshift(~D); % filtrace - dolnofrekvencni propust
imRec=ifft2(imFilt); 
figure; imshow(D); title('Filtr');
figure; imshow(abs(imRec),[]); title('Rekonstruovany obraz po filtraci');
% vysledny obraz byl filtrovan dolni propusti. 
% ZKUSTE SI : zasumte obraz a potom provedte filtraci, provedte filtraci
% hornofrekvencnim filtrem, co ziskate ?  Zkuste si do obrazu pridat mrizku
% a zksute ji pomoci fouriera odstranit.


%========= ROTACE ===================
% Jako posledni priklad si ukazem jak ziskat informaci o geometricke
% strukture obrazu. Orotujem si vstupni obraza zobrazime si jeho fourierovu
% transformaci, kterou na prahujem na zobrazeni 50% maxima
% nacti si obrazek s textem
imR=imrotate(im,45);
figure;imshow(imR);
F_imR=fft2(imR);
ampR=abs(F_imR); 
ampLogR=log(ampR);
figure;imshow(fftshift(ampLogR),[min(ampLogR(:)) max(ampLogR(:))]); title('logaritmicky zobrazene Shifted - Amlitudove spektrum obraz otocen 45'); colormap(jet);
prah=max(ampLogR(:));
imshow(fftshift(ampLogR) > 0.85*prah);

% ZKUSTE : nactete si data pro braillovo pismo a z fourierova obrazu urcete
% natoceni;

